;; Generated package description from elisp-benchmarks.el  -*- no-byte-compile: t -*-
(define-package "elisp-benchmarks" "1.8" "elisp benchmarks collection" 'nil :url "http://elpa.gnu.org/packages/elisp-benchmarks.html" :keywords '("languages" "lisp") :authors '(("Andrea Corallo" . "akrl@sdf.org")) :maintainer '("Andrea Corallo" . "akrl@sdf.org"))
